
# q-expansion of newform 1320.2.a.g, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1320
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [1], [0], [1], [-6], [-6], [0], [0], [-6], [8], [-2], [-2], [-4], [0], [6], [4], [-10], [-12], [0], [-10], [-12], [0], [-6], [10], [10], [-16], [-8], [-10], [2], [16], [-4], [10], [0], [18], [12], [-18], [4], [20], [-18], [-20], [-26], [16], [-10], [6], [8], [16], [-16], [-8], [6], [18], [0], [2], [12], [2], [-4], [6], [28], [2], [14], [-4], [6], [-20], [24], [-22], [30], [12], [-18], [24], [-10], [-14], [-32], [16], [34], [4], [16], [-26], [14], [-6], [18], [-20], [-26], [24], [-6], [12], [-36], [18], [14], [-22], [16], [-36], [0], [16], [-28], [36], [36], [22], [-30], [-28], [22], [28], [22], [-32], [14], [-40], [10], [-12], [-14], [-16], [-30], [0], [-46], [42], [-36], [8], [34], [-28], [-8], [30], [-44], [22], [-42], [-34], [44], [12], [18], [-26], [-16], [-40], [2], [16], [20], [-32], [-10], [-18], [26], [22], [-12], [-2], [-42], [32], [-46], [16], [-8], [-2], [-24], [18], [-30], [44], [32], [-22], [-38], [-4], [12], [-28], [-48], [-52], [-38], [38], [50], [4], [-54], [-24], [-4], [2], [24], [-24], [-14], [18], [54], [28], [-34], [48], [46], [32], [-14], [40], [-50], [24], [30], [0], [20], [22], [-38], [16], [-18], [-46], [-36], [2], [-8], [26], [-24], [-28], [-18], [-32], [42], [-38], [54], [-46], [36], [-30], [44], [-50], [2], [36], [-2], [24], [48], [-42], [20], [-18], [30], [40], [-12], [-32], [-70], [48], [-34], [32], [46], [-10], [-20], [-6], [40], [-24], [-10], [-38], [-16], [-32], [12], [62], [8], [60], [-10], [44], [-36], [-14], [-26], [44], [-48], [68], [32], [-40], [-50], [-54], [-48], [-40], [-44], [-72], [-36], [10], [46], [32], [-6], [6], [60], [-42], [-52], [-26], [54], [-64], [48], [22], [2], [-14], [20], [-26], [18], [4], [-34], [14], [-4], [-46], [-44], [38], [-64], [76], [6], [-62], [-20], [-12], [-64], [-12], [-10], [36], [-24], [-70], [-18], [-64], [-34], [-50], [36], [66], [-76], [-14], [-54], [-80], [-42], [-12], [-36], [22], [-42], [-36], [44], [36], [-46], [-68], [14], [0], [50], [-52], [54], [14], [4], [68], [74], [-12], [-72], [10], [54], [-32], [34], [18], [40], [66], [-62], [44], [-92], [36], [-42], [62], [-34], [4], [24], [80], [28], [62], [-6], [26], [88], [22], [-54], [-30], [24], [14], [-44], [-90], [-52], [-32], [86], [-40], [-86], [70], [16], [-50], [-14], [56], [12], [-62], [32], [26], [-42], [-72], [60], [44], [50], [38], [-16], [34], [12], [-20], [-4], [-6], [-4], [-82], [-36], [48], [62], [6], [46], [30], [-54], [8], [74], [8], [16], [-32], [-42], [-44], [-56], [50], [-10], [60], [84], [-40], [14], [12], [-54], [20], [2], [46], [-14], [24], [90], [2], [-20], [-18], [78], [-76], [68], [-18], [-10], [36], [48], [22], [78], [8], [48], [-94], [36], [-26], [58], [-72], [76], [26], [78], [-84], [22], [-60], [88]]
